using System;
using DotNetLibs.Mail;
using DotNetLibs.Security;

namespace DotNetLibs.Samples
{
	/// <summary>
	/// Provides various helper methods useful for signature validation.
	/// </summary>
	public class ValidationHelper
	{
		/// <summary>
		/// Displays the list of problems of the message that was not validated successfully.
		/// </summary>
		/// <param name="result">Validation result.</param>
		public static void ShowProblems(MailSignatureValidity result)
		{
			Console.WriteLine("There were following problems:");

			string[] missing = result.GetMissingSignatures();
			if (missing.Length > 0)
			{
				Console.WriteLine("- The signatures for the following senders are missing:");
				for (int i=0; i<missing.Length; i++)
					Console.WriteLine("    {0}", missing[i]);
			}

            SysCertValidationStatus certStatus = result.CertificateValidationStatus;
            foreach (SysCertValidationStatus status in Enum.GetValues(typeof(SysCertValidationStatus)))
			{
				if ((status & certStatus) != 0)
					Console.WriteLine("- {0}", GetCertificateSysCertValidationStatusDescription(status));
			}

			SignatureValidationStatus signStatus = result.SignatureValidationStatus;
			foreach (SignatureValidationStatus status in Enum.GetValues(typeof(SignatureValidationStatus)))
			{
				if ((status & signStatus) != 0)
					Console.WriteLine("- {0}", GetSignatureValidationStatusDescription(status));
			}
		}

		/// <summary>
		/// Returns a descriptions of a single SysCertValidationStatus flag.
		/// </summary>
		/// <param name="status">Certificate validation status</param>
		/// <returns>A description.</returns>
		public static string GetCertificateSysCertValidationStatusDescription(SysCertValidationStatus status)
		{
			switch (status)
			{
				case SysCertValidationStatus.TimeNotValid:
					return "Certificate has expired or is not valid yet.";
				case SysCertValidationStatus.Revoked:
					return "Certificate has been revoked.";
				case SysCertValidationStatus.UnknownCa:
					return "Certificate was issued by an unknown authority.";
				case SysCertValidationStatus.RootNotTrusted:
					return "Certificate was issued by an untrusted authority.";
				case SysCertValidationStatus.IncompleteChain:
					return "Certificate does not chain up to a trusted root authority.";
				case SysCertValidationStatus.Malformed:
					return "Certificate is malformed.";
				default:
					return status.ToString();
			}
		}

		/// <summary>
		/// Returns a descriptions of a single SignatureValidationStatus flag.
		/// </summary>
		/// <param name="status">Signature validation status</param>
		/// <returns>A description.</returns>
		public static string GetSignatureValidationStatusDescription(SignatureValidationStatus status)
		{
			switch (status)
			{
				case SignatureValidationStatus.CertificateNotValid:
					return "Certificate is not valid.";
				case SignatureValidationStatus.CertificateNotAvailable:
					return "Certificate is not available.";
				case SignatureValidationStatus.UnsupportedDigestAlgorithm:
					return "A digest algorithm is not supported.";
				case SignatureValidationStatus.UnsupportedSignatureAlgorithm:
					return "A signature algorithm is not supported.";
				case SignatureValidationStatus.InvalidSignature:
					return "A signature is invalid.";
				case SignatureValidationStatus.InvalidKeyUsage:
					return "Invalid key usage.";
				case SignatureValidationStatus.ContentTypeMismatch:
					return "Content type mismatch.";
				default:
					return status.ToString();
			}
		}
	}
}
