﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using DotNetLibs;
using DotNetLibs.Mail;
using DotNetLibs.Net;

namespace ImapConsoleClient
{
    public class ConsoleLogger : BaseVerboseLogger
    {
        public ConsoleLogger(LogLevel level)
        {
            Level = level;
        }

        protected override void WriteMessage(string message)
        {
            Console.WriteLine(message);
        }

        public override void Write(LogLevel level, Type objectType, int objectId, string area, string message)
        {
            if (level < Level)
                return;
            WriteMessage(string.Format("{0}: {1}", area, message));
        }
    }

    class Program
    {
        private static Imap _client;
        private static bool _authenticated;
        private static bool _secured;

        /// <summary>
        /// Show program help.
        /// </summary>
        private static void Help()
        {
            Console.WriteLine("help        connect       disconnect");
            Console.WriteLine("login       head          list");
            Console.WriteLine("get         folders       select");
            Console.WriteLine("examine     unselect      create");
            Console.WriteLine("rename      deletefolder  delete");
            Console.WriteLine("purge       secure        quit");
        }

        /// <summary>
        /// Quit the session.
        /// </summary>
        private static void Quit()
        {
            if (_client.State != ImapState.Disconnected)
            {
                Console.WriteLine("Disconnecting...");
                _client.Disconnect();
            }

            Console.WriteLine();
        }

        static void Main(string[] args)
        {
            // set the trial license key
            DotNetLibs.Licensing.Key = DotNetLibs.Examples.TrialKey.Key;

            while (true)
            {
                Console.Write("imap> ");

                string command = Console.ReadLine().Trim();

                string param = null;

                int i = command.IndexOf(' ');

                if (i > 0)
                {
                    param = command.Substring(i + 1);
                    command = command.Substring(0, i);
                }

                try
                {
                    switch (command.ToLower())
                    {
                        //
                        // Show help
                        //
                        case "?":
                        case "help":
                            Help();
                            break;

                        //
                        // Quit application
                        //
                        case "!":
                        case "bye":
                        case "exit":
                        case "quit":
                            Quit();
                            return;

                        // 
                        // Connect to IMAP server
                        //
                        case "open":
                        case "connect":
                            if (Connect())
                                Login();
                            break;

                        // 
                        // Disconnect
                        //
                        case "close":
                        case "disconnect":
                            Disconnect();
                            break;

                        // 
                        // Secure
                        //
                        case "secure":
                            SecureConnection();
                            break;

                        //
                        // Login
                        //
                        case "login":
                            Login();
                            break;

                        // 
                        // Show message headers
                        //
                        case "head":
                            ShowMessageHeaders(param);
                            break;

                        //
                        // Show message list
                        //
                        case "list":
                            GetMessageList();
                            break;

                        // 
                        // Download message
                        //
                        case "get":
                        case "retr":
                            DownloadMessage(param);
                            break;

                        // 
                        // Show folder list
                        //
                        case "folders":
                            GetFolderList(param);
                            break;

                        // 
                        // Select folder
                        //
                        case "select":
                            SelectFolder(param, false);
                            break;

                        //
                        // Examine
                        //
                        case "examine":
                            SelectFolder(param, true);
                            break;

                        //
                        // Unselect folder
                        //
                        case "unselect":
                            UnselectFolder();
                            break;

                        // 
                        // Create folder
                        // 
                        case "create":
                            CreateFolder(param);
                            break;

                        // 
                        // Rename folder
                        //
                        case "rename":
                            RenameFolder(param);
                            break;

                        // 
                        // Delete folder
                        //
                        case "deletefolder":
                            DeleteFolder(param);
                            break;

                        //
                        // Mark message as deleted
                        //
                        case "del":
                        case "delete":
                            DeleteMessage(param);
                            break;

                        //
                        // Remove deleted message from folder
                        //
                        case "purge":
                            Purge();
                            break;

                        //
                        // Default - invalid command
                        //						
                        default:
                            Console.WriteLine("Invalid command.");
                            break;
                    }
                }
                catch (ArgumentException e)
                {
                    Console.WriteLine(e.Message);
                }
                catch (InvalidOperationException e)
                {
                    Console.WriteLine(e.Message);
                }
                catch (ImapException e)
                {
                    Console.WriteLine(e.Message);
                }
            }
        }

        static bool Connect()
        {
            _client = new Imap();
            _client.LogWriter = new ConsoleLogger(LogLevel.Info);

            Console.Write("IMAP Server: ");
            string server = Console.ReadLine().Trim();

        RetryPort:
            Console.Write("IMAP Port: ");
            int port;
            string s = Console.ReadLine().Trim();
            if (!int.TryParse(s, out port))
            {
                Console.WriteLine("Error: Invalid port");
                goto RetryPort;
            }

            SslMode security;
            Console.Write("Security mode (U)nsecure, (I)mplicit, or (E)xplicit: ");
            s = Console.ReadLine().Trim();
            switch (s.ToLower())
            {
                case "i":
                    security = SslMode.Implicit;
                    break;

                case "e":
                    security = SslMode.Explicit;
                    break;

                default:
                    security = SslMode.None;
                    break;
            }

            _client.Connect(server, port, security);

            _secured = (security != SslMode.None);
            return true;
        }

        static bool CheckConnectionClosed()
        {
            if (_client.State == ImapState.Disconnected)
            {
                Console.WriteLine("The connection was closed.");
                return false;
            }

            return true;
        }

        /// <summary>
        /// Show message list.
        /// </summary>
        private static void GetMessageList()
        {
            ImapMessageCollection messageList;

            if (!CheckConnectionClosed())
                return;

            //
            // get message list
            //
            Console.WriteLine("Getting message list...");

            messageList = _client.GetMessageList();

            if (messageList.Count == 0)
            {
                Console.WriteLine("No messages found.");
                return;
            }

            //
            // show list
            //

            // header
            Console.WriteLine("+--------------------------------------------------------------------------+");
            Console.WriteLine("+   Unique ID         | From                                               +");
            Console.WriteLine("+                     | Subject                                            +");
            Console.WriteLine("+                     | Seq.no  Date                                  Size +");
            Console.WriteLine("+--------------------------------------------------------------------------+");

            // messages
            foreach (ImapMessageInfo message in messageList)
            {
                Console.WriteLine(
                    "| {0} | {1} |\n" +
                    "|                     | {2} |\n" +
                    "|                     | {3}   {4}   {5}B |",

                    message.UniqueId.ToString().PadLeft(19),
                    message.From.ToString().PadRight(50),
                    message.Subject.Substring(0, Math.Min(50, message.Subject.Length)).PadRight(50),
                    message.SequenceNumber.ToString().PadRight(4),
                    message.ReceivedDate.LocalTime.ToString("yyyy-MM-dd HH:mm:ss").PadRight(32),
                    message.Length.ToString().PadLeft(7)
                );
                Console.WriteLine("+--------------------------------------------------------------------------+");
            }
        }

        /// <summary>
        /// Show headers of one message.
        /// </summary>
        /// <param name="uniqueId">Message unique ID.</param>
        private static void ShowMessageHeaders(string uniqueId)
        {
            // check session state
            if (!CheckConnectionClosed())
                return;

            // show help if message unique ID is empty
            if (uniqueId == null || uniqueId.Length == 0)
            {
                Console.WriteLine("Usage: head uniqueID");
                return;
            }

            // download message headers
            ImapMessageInfo messageInfo = _client.GetMessageInfo(uniqueId, ImapListFields.FullHeaders);

            // show headers
            for (int i = 0; i < messageInfo.Headers.Count; i++)
                Console.WriteLine(messageInfo.Headers[i]);
        }

        static void SecureConnection()
        {
            if (!CheckConnectionClosed())
                return;

            if (_authenticated)
            {
                Console.WriteLine("'secure' command is not available after logged in.");
                return;
            }

            if (_secured)
            {
                Console.WriteLine("Connection is already secure.");
                return;
            }

            try
            {
                _client.Secure();
                _secured = true;
                Console.WriteLine("Connection secured!");
            }
            catch (ImapException e)
            {
                if (e.Status == ImapExceptionStatus.ProtocolError)
                {
                    Console.WriteLine("Server does not support SSL/TLS security, disconnecting.");
                }
                Console.WriteLine(e.Message);
                _client.Disconnect();
            }
            catch (TlsException e)
            {
                Console.WriteLine(e.Message);
                Console.WriteLine("TLS negotiation failed, disconnecting.");
                _client.Disconnect();
            }
        }

        static void Disconnect()
        {
            if (!CheckConnectionClosed())
                return;

            _client.Disconnect();
            Console.WriteLine("Disconnected!");
        }

        /// <summary>
        /// Retrieve and display IMAP folder list.
        /// </summary>
        private static void GetFolderList(string folder)
        {
            if (!CheckConnectionClosed())
                return;

            ImapFolderCollection list = _client.GetFolderList(folder);

            foreach (ImapFolder f in list)
            {
                Console.WriteLine("* {0}", f.Name);
            }
        }

        static void DownloadMessageList()
        {
            if (!CheckConnectionClosed())
                return;

            //
            // get message list
            //
            Console.WriteLine("Getting message list...");

            ImapMessageCollection messageList = _client.GetMessageList();

            if (messageList.Count == 0)
            {
                Console.WriteLine("No messages found.");
                return;
            }

            //
            // show list
            //

            // header
            Console.WriteLine("+--------------------------------------------------------------------------+");
            Console.WriteLine("+   Unique ID         | From                                               +");
            Console.WriteLine("+                     | Subject                                            +");
            Console.WriteLine("+                     | Seq.no  Date                                  Size +");
            Console.WriteLine("+--------------------------------------------------------------------------+");

            // messages
            foreach (ImapMessageInfo message in messageList)
            {
                Console.WriteLine(
                    "| {0} | {1} |\n" +
                    "|                     | {2} |\n" +
                    "|                     | {3}   {4}   {5}B |",

                    message.UniqueId.ToString().PadLeft(19),
                    message.From.ToString().PadRight(50),
                    message.Subject.Substring(0, Math.Min(50, message.Subject.Length)).PadRight(50),
                    message.SequenceNumber.ToString().PadRight(4),
                    message.ReceivedDate.LocalTime.ToString("yyyy-MM-dd HH:mm:ss").PadRight(32),
                    message.Length.ToString().PadLeft(7)
                );
                Console.WriteLine("+--------------------------------------------------------------------------+");
            }
        }

        static void DeleteMessage(string uniqueId)
        {
            if (!CheckConnectionClosed())
                return;

            if (uniqueId == null || uniqueId.Length == 0)
            {
                Console.WriteLine("Usage: delete uniqueID");
                return;
            }

            // Mark message as deleted
            _client.DeleteMessage(uniqueId);
        }

        /// <summary>
        /// Delete specified folder.
        /// </summary>
        /// <param name="folderName"></param>
        private static void DeleteFolder(string folderName)
        {
            if (!CheckConnectionClosed())
                return;

            _client.DeleteFolder(folderName);
        }

        /// <summary>
        /// Remove messages marked as deleted.
        /// </summary>
        private static void Purge()
        {
            if (!CheckConnectionClosed())
                return;

            _client.Purge();
        }

        /// <summary>
        /// Retrieve and display IMAP folder list.
        /// </summary>
        private void DownloadFolderList(string folder)
        {
            if (!CheckConnectionClosed())
                return;

            ImapFolderCollection list = _client.GetFolderList(folder);

            foreach (ImapFolder f in list)
            {
                Console.WriteLine("* {0}", f.Name);
            }
        }

        /// <summary>
        /// Select or examine folder.
        /// </summary>
        /// <param name="folderName">Mailbox name.</param>
        /// <param name="readOnly">Whether to select folder in read-only mode.</param>
        private static void SelectFolder(string folderName, bool readOnly)
        {
            if (!CheckConnectionClosed())
                return;

            if (folderName == null || folderName.Length == 0)
            {
                if (readOnly)
                    Console.WriteLine("Usage: examine [foldername]");
                else
                    Console.WriteLine("Usage: select [foldername]");
                return;
            }

            _client.SelectFolder(folderName, readOnly);
            Console.WriteLine("Folder: {0}", _client.CurrentFolder.Name);
            Console.WriteLine("Total messages: {0}", _client.CurrentFolder.TotalMessageCount);
            Console.WriteLine("Unread messages: {0}", _client.CurrentFolder.NotSeenMessageCount);
            Console.WriteLine("Recent messages: {0}", _client.CurrentFolder.RecentMessageCount);
        }

        /// <summary>
        /// Close currently selected folder.
        /// </summary>
        private static void UnselectFolder()
        {
            if (!CheckConnectionClosed())
                return;

            _client.UnselectFolder();
        }

        /// <summary>
        /// Create new folder.
        /// </summary>
        private static void CreateFolder(string foldername)
        {
            if (!CheckConnectionClosed())
                return;

            _client.CreateFolder(foldername);
        }

        /// <summary>
        /// Rename folder.
        /// </summary>
        private static void RenameFolder(string folderName)
        {
            if (!CheckConnectionClosed())
                return;

            string[] names;
            if (folderName != null)
                names = folderName.Split(' ');
            else
                names = null;

            if (names == null || names.Length != 2)
            {
                Console.WriteLine("Usage: rename [current_folder_name] [new_folder_name]");
                return;
            }

            string oldfolderName = names[0];
            string newfolderName = names[1];

            // rename folder
            _client.RenameFolder(oldfolderName, newfolderName);
        }

        /// <summary>
        /// Download one message.
        /// </summary>
        /// <param name="uniqueIdAndPath">Messsage unique ID [path for saving message].</param>
        private static void DownloadMessage(string uniqueIdAndPath)
        {
            if (!CheckConnectionClosed())
                return;

            // check parameters

            string[] p = null;

            if (uniqueIdAndPath != null && uniqueIdAndPath != "")
                p = uniqueIdAndPath.Split(' ');

            if (p == null || p.Length < 0 || p.Length > 2)
            {
                Console.WriteLine("Usage: get uniqueID [path]");
                return;
            }

            if (p.Length == 1)
            {
                // display message to console
                _client.GetMessage(p[0], Console.OpenStandardOutput());
            }
            else
            {
                // download message and save it to file
                string localFilename = p[1];

                string extension = Path.GetExtension(localFilename);
                if (string.Equals(extension, ".msg", StringComparison.OrdinalIgnoreCase))
                {
                    // get message from IMAP (and parse it)
                    MailMessage message = _client.GetMailMessage(p[0]);

                    // save message in MSG format
                    message.Save(localFilename, MailFormat.OutlookMsg);
                }
                else
                {
                    // get message from IMAP directly into file in MIME format (without parsing)
                    _client.GetMessage(p[0], localFilename);
                }
            }
        }

        /// <summary>
        /// Login.
        /// </summary>
        private static void Login()
        {
            // get username
            Console.Write("Username: ");
            string user = Console.ReadLine();

            // get password
            Console.Write("Password: ");
            string pass = Console.ReadLine();

            // login
            _client.Login(user, pass);
            _authenticated = true;


            Console.WriteLine("Logged in successfully.");
            Console.WriteLine("Use 'folders' to get a folder list or 'select Inbox' to select the Inbox folder, or type 'help' for list of supported commands.");
        }
    }
}