using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Net;
using System.Windows.Forms;
using System.Text;

using DotNetLibs;
using DotNetLibs.Net;
using DotNetLibs.Net.Servers;
using DotNetLibs.Samples;

namespace DotNetLibs.Samples
{

	/// <summary>
	/// Main form. Starts the server and shows its UI.
	/// </summary>
	public partial class MainForm : Form
	{
		// sample user credentials
		private string _username = "myuser";
		private string _password = "password";

		// sample user
		private FileServerUser _sampleUser;

		// sample user's virtual root
		private string _userVirtualRoot = ".";

		// server port and 
		private int _port = 2222;
		private FileServer _server;

		// RichTextBox-based ILogWriter
		private RichTextBoxLogWriter _serverLog;

		// server state
		private bool _running;
		private bool _firstTimeStart = true;

		private const string MessageBoxTitle = "Simple File Server (SFTP, SCP, SSH).";

        /// <summary>
        /// Loads and returns the pregenerated private key.
        /// </summary>
        /// <returns>An instance of <see cref="SshPrivateKey"/> class.</returns>
        public static SshPrivateKey GetServerPrivateKey()
        {
            string filePath = AppDomain.CurrentDomain.BaseDirectory + "ServerKey.key";
            if (File.Exists(filePath))
                return new SshPrivateKey(filePath, string.Empty);
            else
            {
                // generate a 1024bit RSA key pair
                SshPrivateKey privateKey = SshPrivateKey.Generate(SshHostKeyAlgorithm.RSA, 1024);

                // save the private key in Base64-encoded PKCS #8 format
                privateKey.Save(filePath, string.Empty, SshPrivateKeyFormat.Pkcs8);

                return privateKey;
            }
        }

		/// <summary>
		/// The MainForm constructor.
		/// </summary>
		public MainForm()
		{
			InitializeComponent();
			CheckForIllegalCrossThreadCalls = true;

			folderBrowserDialog.Description = string.Format("Select the directory that you want to use as the virtual root for '{0}'.", _username);
			_userVirtualRoot = Path.GetFullPath(_userVirtualRoot);
			this.tbxUserRoot.Text = _userVirtualRoot;
		    this.tbxPort.Text = _port.ToString();

			// create RichTextBox-based log writer
			_serverLog = new RichTextBoxLogWriter(tbxLog, tbxLog.MaxLength, LogLevel.Debug); // LogLevel controls the log verbosity

			// set up the server
			try
			{
				_server = new FileServer();
				_server.LogWriter = _serverLog;

				// add private key
				SshPrivateKey rsaKey = GetServerPrivateKey();
				_server.Keys.Add(rsaKey);

				// add a sample user
				_sampleUser = new FileServerUser(_username, _password, _userVirtualRoot);
				_server.Users.Add(_sampleUser);

                _serverLog.WriteToLog("To connect to this DotNetLibs File Server, use an SFTP, SCP or SSH client.\n", Color.Navy);

				btnStartStop.Enabled = true;
				btnBrowseRoot.Enabled = true;
			}
			catch (Exception ex)
			{
				WriteToLog(ex);
			}
		}

		private void BindToPort(int port)
		{
			// allow SFTP subsystem
			_server.Bind(port, FileServerProtocol.Sftp);

			// allow shell-like SCP subsystem
			_server.Bind(port, FileServerProtocol.Scp);
		}

		private int GetPort()
		{
			int result;

			if (!int.TryParse(tbxPort.Text, out result))
				throw new FormatException("The port is not a valid port number.");

			return result;
		}

		/// <summary>
		/// Start the server listeners.
		/// </summary>
		private void StartServer()
		{
			if (_firstTimeStart)
			{
				_port = GetPort();
				BindToPort(_port);
				_firstTimeStart = false;
			}
			else
			{
				// check whether the port has changed
				int newPort = GetPort();
				if (_port != newPort)
				{
					// unbind old port
					_server.Unbind(_port);

					// bind to new port
					BindToPort(newPort);
					_port = newPort;
				}
			}

			_server.Start();
			_running = true;
			this.btnStartStop.Text = "&Stop";
			this.lblServerInfo.Text = string.Format("Running. Listening at port {0}.", _port);

			DisplayConnectionInfo();
		}

		/// <summary>
		/// Stops listening for new connections to the server. (Active connections will be maintained).
		/// </summary>
		private void StopServer()
		{
			_server.Stop();
			_running = false;
			this.btnStartStop.Text = "&Start";
			this.lblServerInfo.Text = "Server has been stopped.";
		}

		/// <summary>
		/// Handles the Start/Stop button click.
		/// </summary>
		private void btnStartStop_Click(object sender, EventArgs e)
		{
			try
			{
				if (!_running)
				{
					StartServer();
				}
				else
				{
					StopServer();
				}
			}
			catch (Exception ex)
			{
				WriteToLog(ex);
			}
		}

		/// <summary>
		/// Stops the server when the form is closed.
		/// </summary>
		private void MainForm_FormClosed(object sender, FormClosedEventArgs e)
		{
			if (_running)
				StopServer();
		}

		/// <summary>
		/// Handles the virtual root change for the sample user.
		/// </summary>
		/// <param name="newPath">The path selected by user.</param>
		private void ChangeVirtualRoot(string newPath)
		{
			if (!Directory.Exists(newPath))
			{
				MessageBox.Show(string.Format("The path could not be found.", newPath), MessageBoxTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
				tbxUserRoot.Text = _userVirtualRoot;
				return;
			}

			newPath = Path.GetFullPath(newPath);

			if (newPath != _userVirtualRoot)
			{
				// update paths
				tbxUserRoot.Text = newPath;
				_userVirtualRoot = newPath;

				// remove the sample user with the old virtual root path
				_server.Users.Remove(_sampleUser);

				// add a new sample user with the new virtual root path
				_sampleUser = new FileServerUser(_username, _password, _userVirtualRoot);
				_server.Users.Add(_sampleUser);


				if (_running)
					MessageBox.Show("The change will apply to new sessions.", MessageBoxTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
				else
					MessageBox.Show("The virtual root has been changed.", MessageBoxTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
			else
			{
				MessageBox.Show(string.Format("The path is the current virtual directory. No change will be performed.", newPath), MessageBoxTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
			}
		}
		
		/// <summary>
		/// Handles the Browse button click.
		/// </summary>
		private void btnBrowseRoot_Click(object sender, EventArgs e)
		{
			DialogResult result = folderBrowserDialog.ShowDialog();
			if (result == DialogResult.OK)
				ChangeVirtualRoot(folderBrowserDialog.SelectedPath);
		}

		/// <summary>
		/// Writes an exception to the log.
		/// </summary>
		/// <param name="x">The exception.</param>
		public void WriteToLog(Exception x)
		{
			_serverLog.WriteToLog("* " + x.ToString(), RichTextBoxLogWriter.COLORERROR);
		}

		/// <summary>
		/// Displays information on how to connect to the server (IP address, port, username, password).
		/// </summary>
		private void DisplayConnectionInfo()
		{
			// use DNS to get the host's IP addresses
			string host = Dns.GetHostName();
			IPHostEntry ip = Dns.GetHostEntry(host);
			StringBuilder ipAddresses = new StringBuilder();
			foreach (IPAddress ipAddress in ip.AddressList)
			{
				if (ipAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
					ipAddresses.AppendFormat("  {0}:{1}\n", ipAddress, _port);
			}

			// display server IP addresses, port, username and password
			_serverLog.WriteToLog(Color.DarkBlue, "\nFile Server has started and listening at:\n{0}\nConnect to the server using the following credentials:\n  Username: {1}\n  Password: {2}\n", ipAddresses, _username, _password);
		}
	}
}
