﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using DotNetLibs.Compression;
using DotNetLibs.FileSystem;

namespace DotNetLibs.Samples
{
	/// <summary>
	/// Simple command line utility that compresses files and/or directories into a ZIP archive.
	/// </summary>
	class Program
	{
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		/// <param name="args">Program arguments.</param>
		static void Main(string[] args)
		{
			// set the trial license key
            DotNetLibs.Licensing.Key = DotNetLibs.Examples.TrialKey.Key;


			if (args.Length == 0)
			{
				ShowHelp();
				return;
			}

			try
			{
				// parse the command line
				Arguments config = new Arguments(args);

				// You can execute the whole operation in a single line of code as follows:
				//	ZipArchive.Add(config.ZipArchivePath, config.SourcePath, null, 0, 0, ActionOnExistingFiles.OverwriteAll);
				
				// If you need to perform multiple ZIP archive operations, 
				// then the following pattern is more efficient:

				// open or create a ZIP archive and add files to it
				using (ZipArchive archive = new ZipArchive(config.ZipArchivePath))
				{
					ArchiveOperationResult result = archive.Add(
						config.SourcePath,
						null,
                        SearchMode.Recursive,
						FileCopyType.Copy,
						OverwriteMode.OverwriteAll);

					Console.WriteLine("Added {0} file(s), {1} byte(s) to {2}.",
						result.FilesAffected,
						result.FilesUncompressedLength,
						archive.FilePath
					);
				}
			}
			catch (SampleException e)
			{
				// display an error reported by arguments parser class
				Console.WriteLine("Error: {0}", e.Message);

				// display help
				ShowHelp();
			}
			catch (ZipException e)
			{
				// display an error reported by the ZIP component
				Console.WriteLine(e.Message);
			}
			catch (DirectoryNotFoundException e)
			{
				// display directory-not-found error
				Console.WriteLine(e.Message);
			}
			catch (FileNotFoundException e)
			{
				// display file-not-found error
				Console.WriteLine(e.Message);
			}
			catch (Exception e)
			{
				// display all other errors
				Console.WriteLine("Error: {0}", e);
			}
		}

		private static void ShowHelp()
		{
			string applicationName = AppDomain.CurrentDomain.FriendlyName;
			Console.WriteLine("=====================================================================");
			Console.WriteLine(" DotNetLibs {0} - Adds files to a ZIP archive", applicationName);
			Console.WriteLine("=====================================================================");
			Console.WriteLine();
			Console.WriteLine("The program is a sample for DotNetLibs ZIP for .NET component.");
			Console.WriteLine("For more info, see http://www.dotnetlibs.com/zip.net/");
			Console.WriteLine();
			Console.WriteLine("Syntax: {0} zip-file-path [path-or-mask]", applicationName);
			Console.WriteLine(@"Example: {0} C:\archive.zip ", applicationName);
			Console.WriteLine(@"Example: {0} C:\archive.zip C:\file.txt", applicationName);
			Console.WriteLine(@"Example: {0} C:\archive.zip C:\data", applicationName);
			Console.WriteLine(@"Example: {0} C:\archive.zip C:\data\*", applicationName);
			Console.WriteLine(@"Example: {0} C:\archive.zip C:\data\*.txt", applicationName);
		}
	}
}
